import React from 'react';
import purebem from 'purebem';
import { Provider } from 'react-redux';
import { mount } from 'enzyme';

import Button from '../button/FoundationButton';
import Textbox from '../textbox/FoundationTextbox';
import TextboxSymbolText from '../textbox/FoundationTextboxSymbolText';

import Autocomplete from './Autocomplete';
import AutocompleteContainer from './AutocompleteContainer';

import autocompleteState from './redux/autocompleteState';

import createStore from '../../test-utils/createStore';


const TestAutocomplete = Autocomplete({
    block: purebem.of('test-autocomplete'),
    displayName: 'TestAutocomplete',
    Button: Button,
    Textbox: Textbox,
    TextboxSymbolText: TextboxSymbolText,
});

const TestAutocompleteContainer = AutocompleteContainer({
    Component: TestAutocomplete,
});

const actionTypePrefix = 'TEST-AUTOCOMPLETE';
const path = 'component';
const root = 'test';

test('should read values from store', () => {
    const options = [
        { value: 'a' },
        { value: 'b' },
    ];

    const store = createStore({
        [root]: {
            [path]: autocompleteState({
                expanded: true,
                fetching: true,
                highlightedIndex: 1,
                options,
                value: 'b',
            }),
        },
    });

    const provider = mount(
        <Provider store={ store }>
            <TestAutocompleteContainer
                actionTypePrefix={ actionTypePrefix }
                path={ path }
                root={ root } />
        </Provider>
    );

    const component = provider.find(TestAutocomplete);
    expect(component.prop('expanded')).toBe(true);
    expect(component.prop('fetching')).toBe(true);
    expect(component.prop('highlightedIndex')).toBe(1);
    expect(component.prop('options')).toEqual(options);
    expect(component.prop('value')).toBe('b');
});
