import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import { connect } from 'react-redux';
import invariant from 'invariant';

import actionName from '../../utils/actionName';
import getComponentState from '../../utils/getComponentState';

import autocompleteChange from './redux/autocompleteChange';
import autocompleteCollapse from './redux/autocompleteCollapse';
import autocompleteDataFetched from './redux/autocompleteDataFetched';
import autocompleteDataFetching from './redux/autocompleteDataFetching';
import autocompleteHighlightedIndexChange from './redux/autocompleteHighlightedIndexChange';


export const changeType = actionName('CHANGE');
export const collapseType = actionName('COLLAPSE');
export const dataFetchedType = actionName('DATA-FETCHED');
export const dataFetchingType = actionName('DATA-FETCHING');
export const highlightedIndexChangeType = actionName('HIGHLIGHTED-INDEX-CHANGE');

export default ({
    Component,
} = {}) => {

    invariant(Component !== undefined, 'Missing `Component`.');

    const Container = React.createClass({

        displayName: `${Component.displayName}Container`,

        mixins: [PureRenderMixin],

        propTypes: {
            actionTypePrefix: PropTypes.string,
            componentRef: PropTypes.func,
            customActionMetadata: PropTypes.object,
            dataProvider: PropTypes.func,
            path: PropTypes.string,
            root: PropTypes.string,
        },

        getDefaultProps() {
            return {
                componentRef: null,
            };
        },

        render() {
            const {
                /* eslint-disable no-unused-vars */
                actionTypePrefix,
                componentRef,
                customActionMetadata,
                dataProvider,
                path,
                root,
                /* eslint-enable no-unused-vars */
                ...otherProps
            } = this.props;

            return (
                <Component
                    { ...otherProps }
                    ref={ componentRef } />
            );
        },

    });

    const mapStateToProps = (state, props) => {
        const { rawValue, ...rest } = getComponentState(state, props);
        return {
            ...rest,
            value: rawValue,
        };
    };

    const mapDispatchToProps = (dispatch, props) => {
        const { customActionMetadata: custom, path, root } = props;
        const actionTypePrefix = props.actionTypePrefix || 'AUTOCOMPLETE';
        const dataProvider = props.dataProvider || (() => Promise.resolve([]));

        return {
            onChange: ({ rawValue, value }) => dispatch(autocompleteChange({ custom, path, rawValue, root, type: changeType(actionTypePrefix), value })),
            onCollapse: () => dispatch(autocompleteCollapse({ path, root, type: collapseType(actionTypePrefix) })),
            onDataFetch: (value) => {
                dispatch(autocompleteDataFetching({ path, root, type: dataFetchingType(actionTypePrefix) }));
                dataProvider(value).then(({ options, optionsTotal }) => dispatch(autocompleteDataFetched({ options, optionsTotal, path, root, type: dataFetchedType(actionTypePrefix) })));
            },
            onHighlightedIndexChange: (highlightedIndex) => dispatch(autocompleteHighlightedIndexChange({ highlightedIndex, path, root, type: highlightedIndexChangeType(actionTypePrefix) })),
        };
    };

    return connect(mapStateToProps, mapDispatchToProps)(Container);

};
