import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isNullOrEmpty from '../../utils/isNullOrEmpty';


export default ({
    block,
    displayName,
    Message,
}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            hasNavigation: PropTypes.bool,
            messages: PropTypes.arrayOf(
                PropTypes.shape({
                    text: PropTypes.string,
                    timeoutId: PropTypes.number,
                    timestamp: PropTypes.number,
                    ui: PropTypes.any,
                })
            ),
            onMessageHide: PropTypes.func,
        },

        getDefaultProps() {
            return {
                hasNavigation: true,
                messages: [],
                onMessageHide: (/* { timeoutId, timestamp } */) => {},
            };
        },

        renderMessage(message) {
            const { onMessageHide } = this.props;
            const { text, timeoutId, timestamp, ui } = message;

            return (
                <Message
                    key={ timestamp }
                    onHide={ onMessageHide }
                    text={ text }
                    timeoutId={ timeoutId }
                    timestamp={ timestamp }
                    ui={ ui } />
            );
        },

        render() {
            const { hasNavigation: navigation, messages } = this.props;
            return (
                <div className={ block({ navigation }) }>
                    { [].map.call(messages, this.renderMessage) }
                </div>
            );
        },

    });

};
