import moment from 'moment';
import formatRelativeDatetime from './formatRelativeDatetime';


test('default same day', () => {
    const timestamp = moment().startOf('day').format('x');
    const date = formatRelativeDatetime(timestamp);

    expect(date).toBe('idag');
});

test('default same day with time', () => {
    const timestamp = moment().startOf('day').format('x');
    const date = formatRelativeDatetime(timestamp, { includeTime: true });

    expect(date).toBe('idag 00:00');
});

test('default next day', () => {
    const timestamp = moment().add(1, 'day').startOf('day').format('x');
    const date = formatRelativeDatetime(timestamp);

    expect(date).toBe('imorgon');
});

test('default next day with time', () => {
    const timestamp = moment().add(1, 'day').startOf('day').format('x');
    const date = formatRelativeDatetime(timestamp, { includeTime: true });

    expect(date).toBe('imorgon 00:00');
});

test('default last day', () => {
    const timestamp = Number(moment().subtract(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp);

    expect(date).toBe('igår');
});

test('default last day with time', () => {
    const timestamp = Number(moment().subtract(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { includeTime: true });

    expect(date).toBe('igår 00:00');
});

test('english same day', () => {
    const timestamp = Number(moment().startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { locale: 'en' });

    expect(date).toBe('today');
});

test('english same day with time', () => {
    const timestamp = Number(moment().startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { includeTime: true, locale: 'en' });

    expect(date).toBe('today 00:00');
});

test('default next day', () => {
    const timestamp = Number(moment().add(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { locale: 'en' });

    expect(date).toBe('tomorrow');
});

test('default next day with time', () => {
    const timestamp = Number(moment().add(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { includeTime: true, locale: 'en' });

    expect(date).toBe('tomorrow 00:00');
});

test('default last day', () => {
    const timestamp = Number(moment().subtract(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { locale: 'en' });

    expect(date).toBe('yesterday');
});

test('default last day with time', () => {
    const timestamp = Number(moment().subtract(1, 'day').startOf('day').format('x'));
    const date = formatRelativeDatetime(timestamp, { includeTime: true, locale: 'en' });

    expect(date).toBe('yesterday 00:00');
});

test('reference', () => {
    const timestamp = Number(moment().startOf('day').format('x'));
    const reference = Number(moment().startOf('day').subtract(1, 'day').format('x'));
    const date = formatRelativeDatetime(timestamp, { reference });

    expect(date).toBe('imorgon');
});
